@@ -1,13 +1,10 @@
-"""
-All atom detection is done here
-
-Everything is in unit of pixel!!
-"""
+#
 # All atom detection is done here
 # Everything is in unit of pixel!!
 #
 
 import numpy as np
+import sys
 
 from skimage.feature import peak_local_max
 from sklearn.cluster import KMeans
@@ -20,9 +17,7 @@ from .file_tools import *
 
 def find_atoms(image, tags):
     """
-    Find atoms
-
-    Not changed to sidpy dataset yet.
+    Find atoms - old please do not use
     """
     image = image - image.min()
     image = image / image.max()
@@ -91,14 +86,7 @@ def find_atoms(image, tags):
 
 def atoms_clustering(atoms, mid_atoms, number_of_clusters=3, nearest_neighbours=7):
     """
-    KMeans clustering of atoms according to nearest neighbour distances
-
-    Input:
-        atoms: (nx2) numpy array or list of floats with atom positions
-        mid-atoms: list of indices of atoms not at rim
-    Optional:
-        number_of_clusters: number of output clusters (default=3)
-        nearest_neighbours: number of nearest neighbours to be considered (default=7)
+    A wrapper for scipy kmeans clustering of atoms.
     """
     # get distances
     nn_tree = cKDTree(np.array(atoms)[:, 0:2])
@@ -114,9 +102,16 @@ def atoms_clustering(atoms, mid_atoms, number_of_clusters=3, nearest_neighbours=
 
 def gauss_difference(params, area):
     """
-    difference between Gaussian and area in image
+    Difference between part of an image and a Gaussian
+    This function is used int he atom refine function of pyTEMlib
+
+    Input:
+    params: list of Gaussian parameters [width, position_x, position_y, intensity]
+    area:  numpy array of 2D matrix = part of an image
+
+    Output:
+    numpy array: flattened array of difference
 
-    used in atom_refine function
     """
     gauss = make_gauss(area.shape[0], area.shape[1], width=params[0], x0=params[1], y0=params[2], intensity=params[3])
     return (area - gauss).flatten()
@@ -124,7 +119,20 @@ def gauss_difference(params, area):
 
 def atom_refine(image, atoms, radius, max_int=0, min_int=0, max_dist=4):
     """
-        fits a Gaussian in a blob
+        fits a Gaussian in a blob of an image
+
+    Input:
+    image: np.array or sidpy Dataset
+    atoms: positions of atoms
+    radius: radius of circular mask to define fitting of Gaussian
+
+    optional
+    max_int: maximum intensity to be considered for fitting (to exclude contaminated areas for example)
+    min_int: minimum intensity to be considered for fitting (to exclude contaminated holes for example)
+    max_dist: maximum distance of movement of Gaussian during fitting
+
+    Output:
+    dictionary: contains new atom positions and other output such as intensity of the fitted Gaussian
     """
     rr = int(radius + 0.5)  # atom radius
     print('using radius ', rr, 'pixels')
@@ -146,13 +154,16 @@ def atom_refine(image, atoms, radius, max_int=0, min_int=0, max_dist=4):
     gauss_amplitude = []
     gauss_intensity = []
     if QT_available:
-        progress = pyTEMlib.sidpy_tools.ProgressDialog("Refine Atom Positions", len(atoms))
+        progress = QtWidgets.QProgressDialog("Refine Atom Positions", "Abort", 0, len(atoms))
+        progress.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
+        # progress.setWindowModality(Qt.WindowModal);
+        progress.show()
 
     done = 0
     for i in range(len(atoms)):
         if QT_available:
-            progress.set_value(i)
-
+            progress.setValue(i)
+            progress.processEvents()
         else:
             if done < int((i + 1) / len(atoms) * 50):
                 done = int((i + 1) / len(atoms) * 50)
@@ -190,7 +201,7 @@ def atom_refine(image, atoms, radius, max_int=0, min_int=0, max_dist=4):
             if (x - rr) < 0 or y - rr < 0 or x + rr + 1 > image.shape[0] or y + rr + 1 > image.shape[1]:
                 pass
             else:
-                [pout, _] = optimization.leastsq(gauss_difference, np.array(guess), args=area)
+                [pout, _] = optimization.leastsq(gauss_difference, guess, args=(area, area))
 
             if (abs(pout[1]) > max_dist) or (abs(pout[2]) > max_dist):
                 pout = [0, 0, 0, 0]
@@ -205,8 +216,6 @@ def atom_refine(image, atoms, radius, max_int=0, min_int=0, max_dist=4):
             gauss_intensity.append((gauss * mask).sum())
         gauss_width.append(pout[0])
         gauss_amplitude.append(pout[3])
-    if QT_available:
-        progress.close()
 
     sym['inside'] = position
     sym['intensity_area'] = intensities
@@ -220,13 +229,10 @@ def atom_refine(image, atoms, radius, max_int=0, min_int=0, max_dist=4):
     return sym
 
 
-def intensity_area(image, atoms, radius=3):
+def intensity_area(image, atoms, radius):
     """
-    Intensity of atoms
-
-    Intensity of atoms in an image within a disk of radius
+    integrated intensity of atoms in an image with a mask around each atom of radius radius
     """
-
     rr = int(radius + 0.5)  # atom radius
     print('using radius ', rr, 'pixels')
 
