@@ -1,6 +1,3 @@
-"""
-EELS_Dialog for interactive_eels
-"""
 from PyQt5 import QtCore,  QtWidgets
 from pyTEMlib import eels_dlg
 
