@@ -1,13 +1,11 @@
-"""
-Image Processing and Analysis Tools of pyTEMlib
-"""
+##################################
 #
 # image_tools.py
 # by Gerd Duscher, UTK
 # part of pyTEMlib
 # MIT license except where stated differently
 #
-
+###############################
 import numpy as np
 
 import matplotlib as mpl
@@ -17,6 +15,7 @@ from matplotlib.patches import Polygon  # plotting of polygons -- graph rings
 import matplotlib.widgets as mwidgets
 from matplotlib.widgets import RectangleSelector
 
+import sidpy
 from .file_tools import *
 from .probe_tools import *
 import sys
@@ -58,15 +57,17 @@ except ModuleNotFoundError:
 if not _SimpleITK_present:
     print('SimpleITK not installed; Registration Functions for Image Stacks not available')
 
-sys.path.insert(0,'../../sidpy')
+
 # Wavelength in 1/nm
 def get_wavelength(e0):
     """
     Calculates the relativistic corrected de Broglie wave length of an electron
 
     Input:
+    ------
         acceleration voltage in volt
     Output:
+    -------
         wave length in 1/nm
     """
 
@@ -74,48 +75,20 @@ def get_wavelength(e0):
     return const.h/np.sqrt(2*const.m_e*eV*(1+eV/(2*const.m_e*const.c**2)))*10**9
 
 
-def read_dm3_image_info(original_metadata):
-    if not isinstance(original_metadata, dict):
-        raise TypeError('We need a python dictionary to read the original metadata')
-    if 'DM' not in original_metadata:
-        return {}
-    main_image = original_metadata['DM']['chosen_image']
-    exp_dictionary = original_metadata['ImageList'][str(main_image)]['ImageTags']
-    experiment = {}
-
-    if 'Acquisition' in exp_dictionary:
-        if 'Parameters' in exp_dictionary['Acquisition']:
-            if 'High Level' in exp_dictionary['Acquisition']['Parameters']:
-                if 'Exposure (s)' in exp_dictionary['Acquisition']['Parameters']['High Level']:
-                    experiment['exposure_time'] = \
-                        exp_dictionary['Acquisition']['Parameters']['High Level']['Exposure (s)']
-
-    if 'Microscope Info' in exp_dictionary:
-        if 'Microscope' in exp_dictionary['Microscope Info']:
-            experiment['microscope'] = exp_dictionary['Microscope Info']['Microscope']
-        if 'Voltage' in exp_dictionary['Microscope Info']:
-            experiment['acceleration_voltage'] = exp_dictionary['Microscope Info']['Voltage']
-        if 'Illumination Mode' in exp_dictionary['Microscope Info']:
-            if exp_dictionary['Microscope Info']['Illumination Mode'] == 'TEM':
-                experiment['convergence_angle'] = 0.0
-                experiment['collection_angle'] = 100.0
-            if exp_dictionary['Microscope Info']['Illumination Mode'] == 'SPOT':
-                experiment['convergence_angle'] = 20.0
-                experiment['collection_angle'] = 50.0
-
-    return experiment
-
 def fourier_transform(dset):
     """
         Reads information into dictionary 'tags', performs 'FFT', and provides a smoothed FT and reciprocal
         and intensity limits for visualization.
 
-        Input:
-            dset: sidp Dataset
+        Input
+        -----
+        dset: sidp Dataset
 
-        Usage:
-            fft_dataset = fourier_transform(sidpy_dataset)
-            fft+dataset.plot()
+        Usage
+        -----
+
+        fft_dataset = fourier_transform(sidpy_dataset)
+        fft+dataset.plot()
     """
 
     assert isinstance(dset, sidpy.Dataset), 'Expected a sidpy Dataset'
@@ -123,7 +96,7 @@ def fourier_transform(dset):
     selection = []
     image_dim = []
     # image_dim = get_image_dims(sidpy.DimensionTypes.SPATIAL)
-    if dset.data_type.name == 'IMAGE_STACK':
+    if dset.data_type == sidpy.DataTypes.IMAGE_STACK:
         for dim, axis in dset._axes.items():
             if axis.dimension_type == sidpy.DimensionTypes.SPATIAL:
                 selection.append(slice(None))
@@ -137,7 +110,7 @@ def fourier_transform(dset):
             raise ValueError('need at least two SPATIAL dimension for an image stack')
         image_stack = np.squeeze(np.array(dset)[selection])
         image = np.sum(np.array(image_stack), axis=stack_dim)
-    elif dset.data_type.name == 'IMAGE':
+    elif dset.data_type == sidpy.DataTypes.IMAGE:
         image = np.array(dset)
     else:
         return
@@ -145,7 +118,7 @@ def fourier_transform(dset):
     image = image - image.min()
     fft_transform = (np.fft.fftshift(np.fft.fft2(image)))
 
-    image_dims = pyTEMlib.sidpy_tools.get_image_dims(dset)
+    image_dims = get_image_dims(dset)
     extent = dset.get_extent(image_dims)
     scale_x = 1 / abs(extent[1] - extent[0])
     scale_y = 1 / abs(extent[2] - extent[3])
@@ -174,15 +147,18 @@ def power_spectrum(dset, smoothing=3):
     Calculate power spectrum
 
     Input:
-        channel: channel in h5f file with image content
+    ------
+            channel: channnel in h5f file with image content
+
     Output:
-        tags: dictionary with
-            ['data']: fourier transformed image
-            ['axis']: scale of reciprocal image
-            ['power_spectrum']: power_spectrum
-            ['FOV']: field of view for extent parameter in plotting
-            ['minimum_intensity']: suggested minimum intensity for plotting
-            ['maximum_intensity']: suggested maximum intensity for plotting
+    -------
+            tags: dictionary with
+                ['data']: fourier transformed image
+                ['axis']: scale of reciprocal image
+                ['power_spectrum']: power_spectrum
+                ['FOV']: field of view for extent parameter in plotting
+                ['minimum_intensity']: suggested minimum intensity for plotting
+                ['maximum_intensity']: suggested maximum intensity for plotting
 
     """
     fft_transform = fourier_transform(dset)
@@ -217,20 +193,23 @@ def diffractogram_spots(dset, spot_threshold):
     Find spots in diffractogram and sort them by distance from center
 
     Input:
-        fft_tags: dictionary with
-            ['spatial_***']: information of scale of fourier pattern
-            ['data']: power_spectrum
-        spot_threshold: threshold for blob finder
+    ------
+            fft_tags: dictionary with
+                ['spatial_***']: information of scale of fourier pattern
+                ['data']: power_spectrum
+            spot_threshold: threshold for blob finder
     Output:
-        spots: numpy array with sorted position (x,y) and radius (r) of all spots
+    -------
+            spots: numpy array with sorted position (x,y) and radius (r) of all spots
+
     """
     # Needed for conversion from pixel to Reciprocal space
     # we'll have to switch x- and y-coordinates due to the differences in numpy and matrix
     center = np.array([int(dset.shape[0]/2.), int(dset.shape[1]/2.), 1])
     rec_scale = np.array([get_slope(dset.u.values), get_slope(dset.v.values), 1])
 
-    # spot detection ( for future reference there is no symmetry assumed here)
-    data = np.abs(dset).T
+    # spot detection ( for future referece there is no symmetry assumed here)
+    data = np.array(dset).T
     data = (data - data.min())
     data = data/data.max()
     # some images are strange and blob_log does not work on the power spectrum
@@ -239,7 +218,7 @@ def diffractogram_spots(dset, spot_threshold):
     except ValueError:
         spots_random = (peak_local_max(np.array(data.T), min_distance=3, threshold_rel=spot_threshold) - center[:2]) \
                        * rec_scale
-        spots_random = np.hstack(spots_random, np.zeros((spots_random.shape[0],1)))
+        spots_random = np.hstack(spots_random,np.zeros((spots_random.shape[0],1)))
 
     print(f'Found {spots_random.shape[0]} reflections')
 
@@ -257,14 +236,16 @@ def adaptive_fourier_filter(dset, spots, low_pass=3, reflection_radius=0.3):
     Use spots in diffractogram for a Fourier Filter
 
     Input:
-        image:  image to be filtered
-        tags: dictionary with
-            ['spatial_***']: information of scale of fourier pattern
-            ['spots']: sorted spots in diffractogram in 1/nm
-        low_pass: low pass filter in center of diffractogram
+    ------
+            image:  image to be filtered
+            tags: dictionary with
+                ['spatial_***']: information of scale of fourier pattern
+                ['spots']: sorted spots in diffractogram in 1/nm
+            low_pass: low pass filter in center of diffractogrm
 
     Output:
-        sidpy dataset or Fourier filtered image
+    -------
+            Fourier filtered image
     """
     # prepare mask
 
@@ -295,9 +276,6 @@ def adaptive_fourier_filter(dset, spots, low_pass=3, reflection_radius=0.3):
 
 
 def rotational_symmetry_diffractogram(spots):
-    """
-    Determine rotational symmetry of diffraction spots
-    """
     rotation_symmetry = []
     for n in [2, 3, 4, 6]:
         cc = np.array(
@@ -313,28 +291,14 @@ def rotational_symmetry_diffractogram(spots):
             rotation_symmetry.append(n)
     return rotation_symmetry
 
-# ####################################################
+#####################################################
 # Registration Functions
-# ####################################################
+#####################################################
 
 
-def complete_registration(main_dataset, storage_channel=None):
-    """
-    First Rigid and the Non-Rigid Registration of image Stack
-
-    Input:
-        sidpy dataset
-    Returns:
-        two sidpy datasets, non-rigid registered first
-    """
+def complete_registration(main_dataset):
     rigid_registered_dataset = rigid_registration(main_dataset)
-    if storage_channel is None:
-        current_channel = main_dataset.h5_dataset.parent
-    else:
-        if not isinstance(storage_channel, h5py.Group):
-            raise ValueError('storage channel needs to be a h5py Group')
-        current_channel = storage_channel
-
+    current_channel = main_dataset.h5_dataset.parent
     registration_channel = log_results(current_channel, rigid_registered_dataset)
 
     print('Non-Rigid_Registration')
@@ -349,14 +313,15 @@ def complete_registration(main_dataset, storage_channel=None):
 def demon_registration(dataset, verbose=False):
     """
     Diffeomorphic Demon Non-Rigid Registration
-
     Usage:
-        dem_reg = demon_reg(cube, verbose = False)
+    ------
+    dem_reg = demon_reg(cube, verbose = False)
 
     Input:
         cube: stack of image after rigid registration and cropping
     Output:
         dem_reg: stack of images with non-rigid registration
+
     Depends on:
         simpleITK and numpy
 
@@ -391,7 +356,7 @@ def demon_registration(dataset, verbose=False):
     done = 0
 
     if QT_available:
-        progress = pyTEMlib.sidpy_tools.ProgressDialog("Non-Rigid Registration", nimages)
+        progress = ProgressDialog("Non-Rigid Registration", nimages)
     for i in range(nimages):
         if QT_available:
             progress.set_value(i)
@@ -435,14 +400,13 @@ def demon_registration(dataset, verbose=False):
 def rigid_registration(dataset):
     """
     Rigid registration of image stack with sub-pixel accuracy
-
-    uses phase_cross_correlation from skimage.registration
+    used phase_cross_correlation from skimage.registration
     (we determine drift from one image to next)
 
-    Input:
-        hdf5 group with image_stack dataset
-    Output:
-        registered stack and drift (with respect to center image)
+    Input hdf5 group with image_stack dataset
+
+    Output Registered Stack and drift (with respect to center image)
+
     """
 
     nopix = dataset.shape[1]
@@ -458,7 +422,7 @@ def rigid_registration(dataset):
     done = 0
 
     if QT_available:
-        progress = pyTEMlib.sidpy_tools.ProgressDialog("Rigid Registration", nimages)
+        progress = ProgressDialog("Rigid Registration", nimages)
     for i in range(nimages):
         if QT_available:
             progress.set_value(i)
@@ -500,14 +464,12 @@ def rig_reg_drift(dset, rel_drift):
     Uses relative drift to shift images ontop of each other
     Shifting is done with shift routine of ndimage from scipy
 
-    is used by rigid_registration routine
+    is used by Rigid_Registration routine
 
-    Input:
-        image_channel with image_stack numpy array
-        relative_drift from image to image as list of [shiftx, shifty]
+    Input image_channel with image_stack numpy array
+    relative_drift from image to image as list of [shiftx, shifty]
 
-    output:
-        stack and drift
+    output stack and drift
     """
 
     rig_reg = np.zeros(dset.shape)
@@ -528,7 +490,7 @@ def rig_reg_drift(dset, rel_drift):
 
 def crop_image_stack(rig_reg, drift):
     """
-    Crop images
+    ## Crop images
     """
     xpmin = int(-np.floor(np.min(np.array(drift)[:, 0])))
     xpmax = int(rig_reg.shape[1] - np.ceil(np.max(np.array(drift)[:, 0])))
@@ -539,9 +501,6 @@ def crop_image_stack(rig_reg, drift):
 
 
 class ImageWithLineProfile:
-    """
-    Just a try
-    """
     def __init__(self, data, extent, title=''):
         fig, ax = plt.subplots(1, 1)
         self.figure = fig
@@ -610,9 +569,6 @@ class ImageWithLineProfile:
         
 
 def histogram_plot(image_tags):
-    """
-    Interactive histogram
-    """
     nbins = 75
     minbin = 0.
     maxbin = 1.
@@ -699,9 +655,6 @@ def histogram_plot(image_tags):
 
 
 def clean_svd(im, pixel_size=1, source_size=5):
-    """
-    Cleans image through svd
-    """
     patch_size = int(source_size/pixel_size)
     if patch_size < 3:
         patch_size = 3
@@ -724,9 +677,12 @@ def rebin(im, binning=2):
     rebin an image by the number of pixels in x and y direction given by binning
     
     Input:
-        image: numpy array in 2 dimensions
+    ------
+            image: numpy array in 2 dimensions
+    
     Output:
-        binned image
+    -------
+            binned image 
     """
     if len(im.shape) == 2:
         return im.reshape((im.shape[0]//binning, binning, im.shape[1]//binning, binning)).mean(axis=3).mean(1)
@@ -736,18 +692,12 @@ def rebin(im, binning=2):
 
 
 def cart2pol(points):
-    """
-    Cartesian to polar coordinates
-    """
     rho = np.linalg.norm(points[:, 0:2], axis=1)
     phi = np.arctan2(points[:, 1], points[:, 0])
     return rho, phi
 
 
 def pol2cart(rho, phi):
-    """
-    Polar to cartesian coordinates
-    """
     x = rho * np.cos(phi)
     y = rho * np.sin(phi)
     return x, y
@@ -755,17 +705,16 @@ def pol2cart(rho, phi):
 
 def xy2polar(points, rounding=1e-3):
     """
-    Conversion from cartesian to polar coordinates
+    Conversion from carthesian to polar coordinates
     
     the angles and distances are sorted by r and then phi
     The indices of this sort is also returned
     
-    input: points:
-        numpy array with number of points in axis 0 first two elements in axis 1 are x and y
-    optional:
-        rounding in significant digits
-    returns:
-        r,phi, sorted_indices
+    input points: numpy array with number of points in axis 0 first two elements in axis 1 are x and y
+    
+    optional rounding in significant digits 
+    
+    returns r,phi, sorted_indices
     """
     
     r, phi = cart2pol(points)
@@ -781,9 +730,6 @@ def xy2polar(points, rounding=1e-3):
             
 
 def cartesian2polar(x, y, grid, r, t, order=3):
-    """
-    Cartesian to ploar coordinates
-    """
 
     rr, tt = np.meshgrid(r, t)
 
@@ -800,9 +746,6 @@ def cartesian2polar(x, y, grid, r, t, order=3):
 
 
 def warp(diff, center):
-    """
-    diffraction pattern to polar coordinates
-    """
     # Define original polar grid
     nx = diff.shape[0]
     ny = diff.shape[1]
@@ -821,9 +764,7 @@ def warp(diff, center):
 
 
 def calculate_ctf(wavelength, cs, defocus, k):
-    """
-    Calculate Contrast Transfer Function
-
+    """ Calculate Contrast Transfer Function
     everything in nm
     """
     ctf = np.sin(np.pi*defocus*wavelength*k**2+0.5*np.pi*cs*wavelength**3*k**4)
@@ -833,17 +774,13 @@ def calculate_ctf(wavelength, cs, defocus, k):
 def calculate_scherzer(wavelength, cs):
     """
     Calculate the Scherzer defocus. Cs is in mm, lambda is in nm
-
-    Input and output in nm
+    # EInput and output in nm
     """
     scherzer = -1.155*(cs*wavelength)**0.5  # in m
     return scherzer
 
 
 def calibrate_image_scale(fft_tags, spots_reference, spots_experiment):
-    """
-    Calibrate image scale based on bragg spots in Fourier space
-    """
     gx = fft_tags['spatial_scale_x']
     gy = fft_tags['spatial_scale_y']
 
@@ -864,9 +801,6 @@ def calibrate_image_scale(fft_tags, spots_reference, spots_experiment):
 
 
 def align_crystal_reflections(spots, crystals):
-    """
-    Find rotation between diffraction spots and diffractogram spots
-    """
     crystal_reflections_polar = []
     angles = []
     exp_r, exp_phi = cart2pol(spots)  # just in polar coordinates
@@ -892,40 +826,33 @@ def align_crystal_reflections(spots, crystals):
         
     return crystal_reflections_polar, angles
 
-
 # Deconvolution
-def decon_lr(o_image, probe, verbose=False):
+def decon_lr(o_image, probe, tags, verbose=False):
     """
-    This task generates a restored image from an input image and point spread function (PSF)
+    # This task generates a restored image from an input image and point spread function (PSF) using
+    # the algorithm developed independently by Lucy (1974, Astron. J. 79, 745) and Richardson
+    # (1972, J. Opt. Soc. Am. 62, 55) and adapted for HST imagery by Snyder
+    # (1990, in Restoration of HST Images and Spectra, ST ScI Workshop Proceedings; see also
+    # Snyder, Hammoud, & White, JOSA, v. 10, no. 5, May 1993, in press).
+    # Additional options developed by Rick White (STScI) are also included.
+    #
+    # The Lucy-Richardson method can be derived from the maximum likelihood expression for data
+    # with a Poisson noise distribution. Thus, it naturally applies to optical imaging data such as HST.
+    # The method forces the restored image to be positive, in accord with photon-counting statistics.
+    #
+    # The Lucy-Richardson algorithm generates a restored image through an iterative method. The essence
+    # of the iteration is as follows: the (n+1)th estimate of the restored image is given by the nth estimate
+    # of the restored image multiplied by a correction image. That is,
+    #
+    #                            original data
+    #       image    = image    ---------------  * reflect(PSF)
+    #            n+1        n     image * PSF
+    #                                  n
 
-    using the algorithm developed independently by Lucy (1974, Astron. J. 79, 745) and Richardson
-    (1972, J. Opt. Soc. Am. 62, 55) and adapted for HST imagery by Snyder
-    (1990, in Restoration of HST Images and Spectra, ST ScI Workshop Proceedings; see also
-    Snyder, Hammoud, & White, JOSA, v. 10, no. 5, May 1993, in press).
-    Additional options developed by Rick White (STScI) are also included.
-
-    The Lucy-Richardson method can be derived from the maximum likelihood expression for data
-    with a Poisson noise distribution. Thus, it naturally applies to optical imaging data such as HST.
-    The method forces the restored image to be positive, in accord with photon-counting statistics.
-
-    The Lucy-Richardson algorithm generates a restored image through an iterative method. The essence
-    of the iteration is as follows: the (n+1)th estimate of the restored image is given by the nth estimate
-    of the restored image multiplied by a correction image. That is,
-
-                                original data
-
-            image    = image    ---------------  * reflect(PSF)
-
-                 n+1        n     image * PSF
-
-                                       n
-
-
-    where the *'s represent convolution operators and reflect(PSF) is the reflection of the PSF, i.e.
-    reflect((PSF)(x,y)) = PSF(-x,-y). When the convolutions are carried out using fast Fourier transforms
-    (FFTs), one can use the fact that FFT(reflect(PSF)) = conj(FFT(PSF)), where conj is the complex conjugate
-
-    operator.
+    # where the *'s represent convolution operators and reflect(PSF) is the reflection of the PSF, i.e.
+    # reflect((PSF)(x,y)) = PSF(-x,-y). When the convolutions are carried out using fast Fourier transforms
+    # (FFTs), one can use the fact that FFT(reflect(PSF)) = conj(FFT(PSF)), where conj is the complex conjugate
+    # operator.
     """
 
     if len(o_image) < 1:
@@ -940,25 +867,27 @@ def decon_lr(o_image, probe, verbose=False):
     error = np.ones(o_image.shape, dtype=np.complex64)
     est = np.ones(o_image.shape, dtype=np.complex64)
     source = np.ones(o_image.shape, dtype=np.complex64)
-    source.real = o_image-o_image.min()+1e-9
+    source.real = o_image
 
     response_ft = fftpack.fft2(probe_c)
 
-    ab = o_image.metadata['experiment']
-    if 'convergence_angle' not in ab:
-        ab['convergence_angle'] = 30
-    ap_angle = ab['convergence_angle'] / 1000.0
+    if 'ImageScanned' in tags:
+        ab = tags['ImageScanned']
+    elif 'aberrations' in tags:
+        ab = tags['aberrations']
+    if 'convAngle' not in ab:
+        ab['convAngle'] = 30
+    ap_angle = ab['convAngle'] / 1000.0
 
-    if 'acceleration_voltage' not in ab:
-        e0 = 200000.
-    else:
-        e0 = float(ab['acceleration_voltage'])
+    e0 = float(ab['EHT'])
 
     wl = get_wavelength(e0)
+    ab['wavelength'] = wl
+
     over_d = 2 * ap_angle / wl
 
-    dx = get_slope(o_image.dim_0)
-    dk = 1.0 / float(o_image.dim_0[-1] - o_image.dim_0[0])
+    dx = tags['pixel_size']
+    dk = 1.0 / float(tags['fov'])
     screen_width = 1 / dx
 
     aperture = np.ones(o_image.shape, dtype=np.complex64)
@@ -974,14 +903,15 @@ def decon_lr(o_image, probe, verbose=False):
 
     tp1 = t_xv ** 2 + t_yv ** 2 >= app_ratio ** 2
     aperture[tp1.T] = 0.
+    print(app_ratio, screen_width, dk)
 
     if QT_available:
-        progress = pyTEMlib.sidpy_tools.ProgressDialog("Lucy-Richardson", 100)
+        progress = ProgressDialog("Lucy-Richardson", 100)
 
     # de = 100
     dest = 100
     i = 0
-    while abs(dest) > 0.001:  # or abs(de)  > .025:
+    while abs(dest) > 0.0001:  # or abs(de)  > .025:
         i += 1
         error_old = np.sum(error.real)
         est_old = est.copy()
@@ -1011,7 +941,8 @@ def decon_lr(o_image, probe, verbose=False):
                 count = 0
             progress.set_value(count)
 
-        if i > 200:
+
+        if i > 1000:
             dest = 0.0
             print('terminate')
     if QT_available:
@@ -1020,9 +951,6 @@ def decon_lr(o_image, probe, verbose=False):
     est2 = np.real(fftpack.ifft2(fftpack.fft2(est) * fftpack.fftshift(aperture)))
     # plt.imshow(np.real(np.log10(np.abs(fftpack.fftshift(fftpack.fft2(est)))+1)+aperture), origin='lower',)
     # plt.show()
-    dset =o_image.like_data(est2.real, name='Lucy_Richardson')
-    dset.data_type = 'image'
-    dset.quantity = 'intensity'
-    dset.source = o_image.title
+    print(est2.shape)
+    return est2
 
-    return dset
