@@ -1,7 +1,3 @@
-"""
-PeakDialog for interactive_eels
-"""
-
 from PyQt5 import QtCore,  QtWidgets
 
 import numpy as np
@@ -26,7 +22,7 @@ _version = .001
 
 class PeakFitDialog(QtWidgets.QDialog):
     """
-    EELS Input Dialog for Peak-Fit Analysis
+    EELS Input Dialog for Chemical Analysis
     """
 
     def __init__(self, dataset=None):
